#include "project-defs.h"
#include "fifo-buffer.h"

void fifoClear(FifoState *buffer) {
	buffer->length = 0;
	buffer->rIndex = 0;
	buffer->wIndex = 0;
	buffer->status = 0;
}

bool fifoIsFull(FifoState *fifo) {
	return fifoLength(fifo) == fifo->size;
}

size_t fifoBytesFree(FifoState *fifo) {
	return fifo->size - fifoLength(fifo);
}

bool fifoWrite(FifoState *buffer, const void *data, size_t count) {
	bool rc = fifoBytesFree(buffer) >= count;
	
	if (rc) {
		size_t wIndex = buffer->wIndex;
		
		for (size_t n = 0; n < count; n++) {
			if (wIndex == buffer->size) {
				wIndex = 0;
			}
			
			buffer->data[wIndex] = ((const uint8_t *) data)[n];
			wIndex++;
		}
		
		buffer->wIndex = wIndex;
		buffer->length += count;
	}
	
	return rc;
}

bool fifoRead(FifoState *buffer, void *data, size_t count) {
	bool rc = fifoLength(buffer) >= count;
	
	if (rc) {
		size_t rIndex = buffer->rIndex;
		
		for (size_t n = 0; n < count; n++) {
			if (rIndex == buffer->size) {
				rIndex = 0;
			}
			
			((uint8_t *) data)[n] = buffer->data[rIndex];
			rIndex++;
		}
		
		buffer->rIndex = rIndex;
		buffer->length -= count;
	}
	
	return rc;
}
