#ifndef _UART_API_H
#define _UART_API_H

/**
 * @file uart-api.h
 *
 * Exemple d'abstraction de l'utilisation de l'UART.
 *
 * Si on voulait porter notre application sur un autre micro-contrôleur,
 * il suffirait d'adapter uart-hal.c, tout le code reposant sur
 * l'utilisation de uart-hal.h resterait inchangé.
 */

void uartInitialise(uint32_t baudRate);

/**
 * Envoie un bloc d'octets.
 *
 * @param data Adresse du bloc à envoyer.
 * @param size Taille du bloc à envoyer.
 * @return true si le bloc a été placé dans le tampon d'émission.
 */
bool uartSendBlock(const uint8_t *data, size_t size);

/**
 * Récupère un bloc d'octets du tampon de réception.
 *
 * @param data Adresse du tampon de stockage.
 * @param size Taille du bloc à récupérer.
 * @return true si le tampon de réception contenait suffisamment de données.
 */
bool uartGetBlock(uint8_t *data, size_t size);

/**
 * Détermine si la transmission est terminée.
 *
 * @return true si le tampon d'émission est vide.
 */
bool uartIsTransmissionComplete();

/**
 * Vérifie le nombre d'octets libres dans le tampon d'émission de l'UART.
 *
 * @param bytes Nombre minimum requis.
 * @return true si le tampon d'émission dispose d'au moins 'bytes' octets libres.
 */
bool uartTransmitBufferHasBytesFree(size_t bytes);

/**
 * Détermine le nombre d'octets stockés dans le tampon de réception.
 *
 * @return le nombre d'octets présents dans le tampon de réception de l'UART.
 */
int uartReceiveBufferBytes();

/**
 * Vide le tampon de réception de l'UART.
 */
void uartFlushReceiveBuffer();

#endif // _UART_API_H
